searchData={"items":[{"type":"module","title":"MatchEngine","doc":"MatchEngine is an in-memory matching/filtering engine with\nMongoDB-like query syntax.","ref":"MatchEngine.html"},{"type":"module","title":"Introduction - MatchEngine","doc":"The query language consists of nested Elixir \"keyword list\". Each\ncomponent of the query consists of a _key_ part and a _value_\npart. The key part is either a logic operator (and/or/not), or a\nreference to a field, the value part is either a plain value, or a\nvalue operator.\n\nWhen a query is run against a document, where each term is scored\nindividually and then summed. (This implies \"or\"). Some example\nqueries:\n\n```\n[title: \"hoi\"]\n[title: [_eq: \"hoi\"]]\n[_and: [name: \"John\", age: 36]]\n[_or: [name: \"John\", age: 36]]\n[_not: [title: \"foo\"]]\n```\n\nTwo ways of saying \"Score all documents in which the title equals `\"hoi\"`\":\n\n```\n[title: \"hoi\"]\n[title: [_eq: \"hoi\"]]\n```\n\nCombining various matchers with logic operators:\n\n```\n[_and: [name: \"John\", age: 36]]\n[_or: [name: \"John\", age: 36]]\n[_not: [title: \"foo\"]]\n```\n\nPerforming matches in nested objects is also possible; the query\nsimply follows the shape of the data.\n\nGiven a document consisting of a nested structure, `%{\"user\" => %{\"name\" => \"John\"}}`:\n\n\"User name equals John\":\n\n```\n[user: [name: \"John\"]]\n```\n\n\"User name does not equal John\":\n\n```\n[_not: [user: [name: \"John\"]]]\n```\n\n> Note that this is a different approach for nesting fields than MongoDB, which uses dot notation for field nesting.","ref":"MatchEngine.html#module-introduction"},{"type":"module","title":"Query execution - MatchEngine","doc":"The queries can be run by calling `MatchEngine.score_all/2` or `MatchEngine.filter_all/2`.\n\nQueries are first preprocessed, and then executed on a list of search\n\"documents\". A \"document\" is just a normal Elixir map, with string\nkeys.\n\nThe preprocessing phase compiles any regexes, checks whether all\noperators exist, and de-nests nested field structures.\n\nThe query phase runs the preprocessed query for each document in the\nlist, by calculating the score for the given document, given the\nquery. When using filter_all/2, documents with a zero score are\nremoved from the input list. When using score_all, the list is\nsorted on score, descending, and this score, including any\nadditional metadata, is returned in a `\"_match\"` map inside the\ndocument.","ref":"MatchEngine.html#module-query-execution"},{"type":"module","title":"Value operators - MatchEngine","doc":"_Value operators_ work on an individual field. Various operators can\nbe used to calculate a score for a given field.\n\n### `_eq`\n\nScores on the equality of the argument.\n\n```\n[title: \"hello\"]\n[title: [_eq: \"hello\"]]\n```\n\n### `_ne`\n\nScores on the *in*equality of the argument. (\"Not equals\")\n\n```\n[title: [_ne: \"hello\"]]\n```\n\n### `_has`\n\nScores when the document's value contains a member of the given list or contains the given word or words\n\n```\n[tag: [_has: [\"production\"]]]\n[title: [_has: \"The\"]]\n[title: [_has: [\"The\", \"title\"]]]\n```\n\n### `_hasnt`\n\nScores when the document's value does NOT contains any member of the given list\nor does not contain any of the given word or words\n\n```\n[tag: [_hasnt: [\"production\"]]]\n[title: [_hasnt: \"The\"]]\n[title: [_hasnt: [\"The\", \"title\"]]]\n```\n\n### `_in`\n\nScores when the document's value is a member of the given list.\n\n```\n[role: [_in: [\"developer\", \"freelancer\"]]]\n```\n\n### `_nin`\n\nScores when the document's value is _not_ a member of the given list.\n\n```\n[role: [_nin: [\"recruiter\"]]]\n```\n\n### `_lt`, `_gt`, `_lte`, `_gte`\n\nScores on using the comparison operators <, >, <= and >=.\n\n```\n[age: [_gt: 18]]\n```\n\n### `_sim`\n\nNormalized string similarity. The max of the Normalised Levenshtein\ndistance and Jaro distance.\n\n### `_regex`\n\nMatch a regular expression. The input is a string, which gets compiled\ninto a regex. This operator scores on the length of match divided by\nthe total string length. It is possible to add named captures to the\nregex, which then get added to the `_match` metadata map, as seen in the following example:\n\n```\n# regex matches entire string, 100% score\nassert %{\"score\" => 1} == score([title: [_regex: \"foo\"]], %{\"title\" => \"foo\"})\n# regex matches with a capture called 'name'. It is boosted by weight.\nassert %{\"score\" => 1.6, \"name\" => \"food\"} == score([title: [_regex: \"(?P foo[dl])\", w: 4]], %{\"title\" => \"foodtrucks\"})\n```\n\nThe regex match can also be inversed, where the document value is\ntreated as the regular expression, and the query input is treated as\nthe string to be matched. (No captures are supported in this case).\n\n```\nassert %{\"score\" => 0.5} == score([title: [_regex: \"foobar\", inverse: true]], %{\"title\" => \"foo\"})\n```\n\n### `_geo`\n\nCalculate document score based on its geographical distance to a given\npoint. The geo distance (both in the operator and in the document) can\nbe given as:\n\n- A regular list, e.g. `[4.56, 52.33]`\n- A keyword list, e.g. `[lat: 52.33, lon: 4.56]`\n- A map with atom keys, e.g. `%{lat: 52.33, lon: 4.56}`\n- A map with string keys, e.g. `%{\"lat\" => 52.33, \"lon\" => 4.56}`\n\nThe calculated `distance` is returned in meters, as part of the `_match` map.\n\nAn extra argument, `max_distance` can be given to the operator which\nspecifies the maximum cutoff point. It defaults to 100km. (100_000).\nDistance is scored logarithmically with respect to the maximum\ndistance.\n\n```\ndoc = %{\"location\" => %{\"lat\" => 52.340500999999996, \"lon\" => 4.8832816}}\nq = [location: [_geo: [lat: 52.340500999999996, lon: 4.8832816]]]\nassert %{\"score\" => 1, \"distance\" => 0.0} == score(q, doc)\n```\n\nWhen `radius` is given as an option, all geo points that are within\nthe radius will score a 1 and the max_distance scoring will be in\neffect for distances larger than the radius.\n\n### `_geo_poly`\n\nCalculate document score based on its containment inside a given\ngeographical polygon.\n\nAccepts a list of geographical coordinates, each in the same format\nas `_geo`.\n\nLike `_geo`, the `max_distance` option can be given to the operator\nwhich specifies the maximum cutoff point. It defaults to\n100km. (100_000). Distance is scored logarithmically with respect\nto the maximum distance.\n\nWhen the point is inside the polygon, the score is always 1. Only\nwhen the point is outside the polygon, the geographical distance\nfrom the document point to the closest point on the edge of the\npolygon is calculated and scored based on the `max_distance`\nsetting.\n\n### `_time`\n\nScore by an UTC timestamp, relative to the given time.\n\n```\nt1 = \"2018-02-19T15:29:53.672235Z\"\nt2 = \"2018-02-19T15:09:53.672235Z\"\nassert %{\"score\" => s} = score([inserted_at: [_time: t1]], %{\"inserted_at\" => t2})\n```\n\nThis way, documents can be returned in order of recency.","ref":"MatchEngine.html#module-value-operators"},{"type":"module","title":"Logic operators - MatchEngine","doc":"### `_and`\n\nCombine matchers, multiplying the score. When one of the matchers\nreturns 0, the total score is 0 as well.\n\n```\n[_and: [name: \"John\", age: 36]]\n```\n\n### `_or`\n\nCombine matchers, adding the scores.\n\n```\n[_or: [name: \"John\", id: 12]]\n```\n\n### `_not`\n\nReverse the score of the nested matchers. (when score > 0, return 0, otherwise, return 1.\n\n```\n[_not: [title: \"foo\"]]\n```\n\n#","ref":"MatchEngine.html#module-logic-operators"},{"type":"module","title":"Matcher weights - MatchEngine","doc":"`w: 10` can be added to a matcher term to boost its score by the given weight.\n\n```\n[title: [_eq: \"Pete\", w: 5], summary: [_sim: \"hello\", w: 2]]\n```\n\n`b: true` can be added to force a score of 1 when the score is > 0.\n\n```\n[title: [_sim: \"hello\", b: true]]\n```","ref":"MatchEngine.html#module-matcher-weights"},{"type":"module","title":"Map syntax for queries - MatchEngine","doc":"Instead of keyword lists, queries can also be specified as maps. In\nthis case, the keys of the map need to be strings. Query maps are\nmeant to be used from user-generated input, and can be easily created from JSON files.\n\n```\n[_not: [title: \"foo\"]]\n# can also be written as:\n%{\"_not\" => %{\"title\" => \"foo\"}}\n\n[title: [_eq: \"Pete\", w: 5], summary: [_sim: \"hello\", w: 2]]\n# can also be written as:\n%{\"title\" => %{\"_eq\" => \"Pete\", \"w\" => 5}, \"summary\" => %{\"_sim\" => \"hello\", \"w\" => 2}}\n```","ref":"MatchEngine.html#module-map-syntax-for-queries"},{"type":"function","title":"MatchEngine.filter/2","doc":"Filter a single document agains the given query\n\nTop-level query operators are treated as `and` clauses.  The return\nvalue includes `score` attribute which contains the actual score.","ref":"MatchEngine.html#filter/2"},{"type":"function","title":"MatchEngine.filter_all/2","doc":"Filter all given documents agains the given query.\n\nOnly the documents that have a positive (greater than 0) score are\nreturned. The document order is preserved, no sorting on score is done.","ref":"MatchEngine.html#filter_all/2"},{"type":"function","title":"MatchEngine.score/2","doc":"Score a single document agains the given query\n\nTop-level query operators are treated as `or` clauses.  The return\nvalue includes `score` attribute which contains the actual score.","ref":"MatchEngine.html#score/2"},{"type":"function","title":"MatchEngine.score_all/2","doc":"Score all given documents against the given query.\n\nAll documents are returned, even when their score is 0. The returned\nlist of documents is sorted on their score, descending (best\nmatching document first).\n\nThe document contains a `_match` key which contains the `score`\nattribute. Some operators, e.g. `_geo`, add additional information\nto this match map, for instance, the geographical distance.","ref":"MatchEngine.html#score_all/2"},{"type":"type","title":"MatchEngine.doc/0","doc":"","ref":"MatchEngine.html#t:doc/0"},{"type":"type","title":"MatchEngine.doc_with_match/0","doc":"","ref":"MatchEngine.html#t:doc_with_match/0"},{"type":"type","title":"MatchEngine.operator/0","doc":"","ref":"MatchEngine.html#t:operator/0"},{"type":"type","title":"MatchEngine.operator_arg/0","doc":"","ref":"MatchEngine.html#t:operator_arg/0"},{"type":"type","title":"MatchEngine.operator_pair/0","doc":"","ref":"MatchEngine.html#t:operator_pair/0"},{"type":"type","title":"MatchEngine.query/0","doc":"","ref":"MatchEngine.html#t:query/0"},{"type":"type","title":"MatchEngine.score_match/0","doc":"","ref":"MatchEngine.html#t:score_match/0"},{"type":"module","title":"MatchEngine.Levenshtein","doc":"The implementation of the [Levenshtein](http://en.wikipedia.org/wiki/Levenshtein_distance) distance metric.\n\nInlined from the Simetric library.","ref":"MatchEngine.Levenshtein.html"},{"type":"function","title":"MatchEngine.Levenshtein.compare/2","doc":"Returns an integer representing the minimum number of\nsingle-character edits (i.e. insertions, deletions or substitutions)\nrequired to change `string1` into the `string2`.","ref":"MatchEngine.Levenshtein.html#compare/2"},{"type":"function","title":"Examples - MatchEngine.Levenshtein.compare/2","doc":"iex> MatchEngine.Levenshtein.compare(\"kitten\", \"sitting\")\n    3","ref":"MatchEngine.Levenshtein.html#compare/2-examples"}],"content_type":"text/markdown"}